#! python
import logging
import os
import tempfile
from typing import Optional

import click
import git

from .apis import clipboard
from .apis import git as git_api
from .apis import kak

DEFAULT_LOGFILE = "github-permalink.log"

LOG_LEVELS = {
    "info": logging.INFO,
    "debug": logging.DEBUG,
    "critical": logging.CRITICAL,
}


@click.command()
@click.option(
    "-l",
    "--log-level",
    default="info",
    help="Level of information to write to the logs.",
    type=click.Choice(LOG_LEVELS),
)
@click.option("-p", "--log-path", help="Where to write the logfile.")
def main(log_level: str, log_path: Optional[str]) -> None:
    """Call kcr to get editor info, then parse it and write it to the clipboard"""
    _setup_logging(log_level, log_path)
    get_github_permalink()
    logging.info("=== script complete ===")


def _setup_logging(log_level: str, log_path: Optional[str]) -> None:  # pragma: no cover
    if log_path is None:
        temp_dir = tempfile.gettempdir()
        log_path = f"{temp_dir}/{DEFAULT_LOGFILE}"
    logging.basicConfig(
        filename=log_path,
        format="%(levelname)s: %(message)s",
        level=LOG_LEVELS[log_level],
    )
    logging.info(f"log level set to {log_level.upper()}")


def get_github_permalink() -> None:
    absolute_path, selection_desc = kak.kcr_get(["buffile", "selection_desc"])
    selection = kak.SelectionDescription(selection_desc)
    repo = git_api.RepoApi(git.Repo("."))
    relative_path = os.path.relpath(absolute_path, os.getcwd())
    permalink = repo.get_permalink(relative_path, selection.range)
    clipboard_command = clipboard.get_clipboard_command()
    clipboard.write_to_clipboard(permalink, clipboard_command)



import logging
import os
import pipes

# TODO: how do I test this file??


def get_clipboard_command() -> str:
    clipboard = os.getenv("CLIPBOARD")
    if clipboard is None:
        raise EnvironmentError("CLIPBOARD is not defined")
    else:
        return clipboard


def write_to_clipboard(message: str, clipboard_command: str) -> None:
    logging.debug(f"Clipboard is {clipboard_command}")
    template = pipes.Template()
    # takes in input and writes no output
    template.append(clipboard_command, "-.")
    with template.open("tempfile", "w") as clipboard:  # won't actually open a file
        clipboard.write(message)
    logging.debug(f"writing '{message}' to clipboard")



import json
import logging
import pipes
from typing import List

from .. import line_range


def kcr_get(values: List) -> List:
    # TODO: how do we test this???
    value_flags = [f"--value {value}" for value in values]
    args = " ".join(value_flags)
    command = f"kcr get {args}"
    logging.debug(f"running commmand: '{command}'")
    template = pipes.Template()
    template.prepend(command, ".-")
    with template.open("tempfile", "r") as pipe_outfile:
        kcr_output = pipe_outfile.read()
    logging.debug(f"kcr output: {kcr_output}")
    return json.loads(kcr_output)


class SelectionDescription(object):
    def __init__(self, selection_desc: str) -> None:
        """Parse a selction description from kakoune"""
        anchor_pos, cursor_pos = selection_desc.split(",")
        anchor_line = int(anchor_pos.split(".")[0])
        cursor_line = int(cursor_pos.split(".")[0])
        logging.debug(f"anchor at {anchor_line}; cursor at {cursor_line}")
        self._line_range = line_range.LineRange(
            min(anchor_line, cursor_line), max(anchor_line, cursor_line)
        )

    @property
    def range(self) -> line_range.LineRange:
        return self._line_range
